// -*- C++ -*-
/***************************************************************************
 * blitz/tinyveciter.h   Declaration of TinyVectorIter<T,N,stride>
 *
 * $Id: tinyveciter.h,v 1.7 2011/03/25 22:41:16 julianc Exp $
 *
 * Copyright (C) 1997-2011 Todd Veldhuizen <tveldhui@acm.org>
 *
 * This file is a part of Blitz.
 *
 * Blitz is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Blitz is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with Blitz.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Suggestions:          blitz-devel@lists.sourceforge.net
 * Bugs:                 blitz-support@lists.sourceforge.net    
 *
 * For more information, please see the Blitz++ Home Page:
 *    https://sourceforge.net/projects/blitz/
 *
 ***************************************************************************/


#ifndef BZ_TINYVECITER_H
#define BZ_TINYVECITER_H

#ifndef BZ_TINYVEC_H
 #include <blitz/tinyvec.h>
#endif

#ifndef BZ_MANUAL_VECEXPR_COPY_CONSTRUCTOR
 #error "Debug in tinyveciter.h (this line shouldn't be here)"
#endif

BZ_NAMESPACE(blitz)

// N_stride has default 1, in forward declaration in <blitz/tinyvec.h>
template<typename P_numtype, int N_length, int N_stride>
class TinyVectorIter {
public:
    typedef P_numtype T_numtype;

    explicit TinyVectorIter(TinyVector<T_numtype, N_length>& x)
        : data_(x.data())
    { }

#ifdef BZ_MANUAL_VECEXPR_COPY_CONSTRUCTOR
    TinyVectorIter(const TinyVectorIter<T_numtype, N_length, N_stride>& iter)
        : data_(iter.data_)
    {
    }
#endif

    T_numtype operator[](int i) const
    {
        BZPRECONDITION(i >= 0 && i < N_length);
        return data_[i * N_stride];
    }

    T_numtype& restrict operator[](int i)
    {
        BZPRECONDITION(i >= 0 && i < N_length);
        return data_[i * N_stride];
    }

    T_numtype operator()(int i) const
    {
        BZPRECONDITION(i >= 0 && i < N_length);
        return data_[i * N_stride];
    }

    T_numtype& restrict operator()(int i)
    {
        BZPRECONDITION(i >= 0 && i < N_length);
        return data_[i * N_stride];
    }

    int length(int) const
    { return N_length; }

    static const int _bz_staticLengthCount = 1,
                     _bz_dynamicLengthCount = 0,
                     _bz_staticLength = 0;

    bool _bz_hasFastAccess() const
    { return true; }

    T_numtype _bz_fastAccess(int i) const
    { return data_[i * N_stride]; }

    T_numtype& _bz_fastAccess(int i)
    { return data_[i * N_stride]; }

    int _bz_suggestLength() const
    { return N_length; }

private:
    T_numtype * restrict data_;
};

// N_stride has default 1, in forward declaration in <blitz/tinyvec.h>
template<typename P_numtype, int N_length, int N_stride>
class TinyVectorIterConst {
public:
    typedef P_numtype T_numtype;

    explicit TinyVectorIterConst(const TinyVector<T_numtype, N_length>& x)
        : data_(x.data())
    { }

#ifdef BZ_MANUAL_VECEXPR_COPY_CONSTRUCTOR
    TinyVectorIterConst(const TinyVectorIterConst<T_numtype, N_length, 
        N_stride>& iter)
        : data_(iter.data_)
    {
    }

    void operator=(const TinyVectorIterConst<T_numtype, N_length, N_stride>& 
        iter)
    {
        data_ = iter.data_;
    }
#endif

    T_numtype operator[](int i) const
    {
        BZPRECONDITION(i >= 0 && i < N_length);
        return data_[i * N_stride];
    }

    T_numtype operator()(int i) const
    {
        BZPRECONDITION(i >= 0 && i < N_length);
        return data_[i * N_stride];
    }

    int length(int) const
    { return N_length; }

    static const int _bz_staticLengthCount = 1,
                     _bz_dynamicLengthCount = 0,
                     _bz_staticLength = 0;

    bool _bz_hasFastAccess() const
    { return true; }

    T_numtype _bz_fastAccess(int i) const
    { return data_[i * N_stride]; }

    int _bz_suggestLength() const
    { return N_length; }

private:
    const T_numtype * restrict data_;
};

BZ_NAMESPACE_END

#endif // BZ_TINYVECITER_H
